@@ -8,7 +8,7 @@
  * @version    @version@
  * @author     Uwe Steinmann <uwe@steinmann.cx>
  * @copyright  Copyright (C) 2010, Uwe Steinmann
- * @version    Release: 3.0.1
+ * @version    Release: 3.3.11
  */
 
 /**
@@ -68,7 +68,7 @@ require_once("inc.ClassNotification.php");
  * @version    @version@
  * @author     Uwe Steinmann <uwe@steinmann.cx>
  * @copyright  Copyright (C) 2010, Uwe Steinmann
- * @version    Release: 3.0.1
+ * @version    Release: 3.3.11
  */
 class LetoDMS_Core_DMS {
 	/**
@@ -178,12 +178,13 @@ class LetoDMS_Core_DMS {
 		else
 			$this->contentDir = $contentDir.'/';
 		$this->rootFolderID = 1;
+		$this->maxDirID = 0; //31998;
 		$this->enableAdminRevApp = false;
 		$this->enableConverting = false;
 		$this->convertFileTypes = array();
-		$this->version = '3.0.1';
+		$this->version = '3.3.11';
 		if($this->version[0] == '@')
-			$this->version = '3.0.0';
+			$this->version = '3.3.11';
 	} /* }}} */
 
 	function getDB() { /* {{{ */
@@ -214,13 +215,14 @@ class LetoDMS_Core_DMS {
 	 * Check if the version in the database is the same as of this package
 	 * Only the major and minor version number will be checked.
 	 *
-	 * @return boolean returns false if versions do not match
+	 * @return boolean returns false if versions do not match, but returns
+	 *         true if version matches or table tblVersion does not exists.
 	 */
 	function checkVersion() { /* {{{ */
 		$tbllist = $this->db->TableList();
 		$tbllist = explode(',',strtolower(join(',',$tbllist)));
 		if(!array_search('tblversion', $tbllist))
-			return false;
+			return true;
 		$queryStr = "SELECT * FROM tblVersion order by major,minor,subminor limit 1";
 		$resArr = $this->db->getResultArray($queryStr);
 		if (is_bool($resArr) && $resArr == false)
@@ -245,6 +247,26 @@ class LetoDMS_Core_DMS {
 		$this->rootFolderID = $id;
 	} /* }}} */
 
+	/**
+	 * Set maximum number of subdirectories per directory
+	 *
+	 * The value of maxDirID is quite crucial because, all documents are
+	 * associated with a directory in the filesystem. Consequently, there is
+	 * maximum number of documents, because depending on the file system
+	 * the maximum number of subdirectories is limited. Since version 3.3.0 of
+	 * letodms an additional directory level has been introduced. All documents
+	 * from 1 to maxDirID-1 will be saved in 1/<docid>, documents from maxDirID
+	 * to 2*maxDirID-1 are stored in 2/<docid> and so on.
+	 *
+	 * This function must be called right after creating an instance of
+	 * LetoDMS_Core_DMS
+	 *
+	 * @param interger $id id of root folder
+	 */
+	function setMaxDirID($id) { /* {{{ */
+		$this->maxDirID = $id;
+	} /* }}} */
+
 	/**
 	 * Get root folder
 	 *
@@ -311,7 +333,7 @@ class LetoDMS_Core_DMS {
 	function getDocument($id) { /* {{{ */
 		if (!is_numeric($id)) return false;
 
-		$queryStr = "SELECT * FROM tblDocuments WHERE id = " . $id;
+		$queryStr = "SELECT * FROM tblDocuments WHERE id = " . (int) $id;
 		$resArr = $this->db->getResultArray($queryStr);
 		if (is_bool($resArr) && $resArr == false)
 			return false;
@@ -320,7 +342,7 @@ class LetoDMS_Core_DMS {
 		$resArr = $resArr[0];
 
 		// New Locking mechanism uses a separate table to track the lock.
-		$queryStr = "SELECT * FROM tblDocumentLocks WHERE document = " . $id;
+		$queryStr = "SELECT * FROM tblDocumentLocks WHERE document = " . (int) $id;
 		$lockArr = $this->db->getResultArray($queryStr);
 		if ((is_bool($lockArr) && $lockArr==false) || (count($lockArr)==0)) {
 			// Could not find a lock on the selected document.
@@ -377,7 +399,7 @@ class LetoDMS_Core_DMS {
 		$queryStr = "SELECT `tblDocuments`.*, `tblDocumentLocks`.`userID` as `lockUser` ".
 			"FROM `tblDocuments` ".
 			"LEFT JOIN `tblDocumentLocks` ON `tblDocuments`.`id`=`tblDocumentLocks`.`document` ".
-			"WHERE `tblDocuments`.`name` = '" . $name . "'";
+			"WHERE `tblDocuments`.`name` = " . $this->db->qstr($name);
 		if($folder)
 			$queryStr .= " AND `tblDocuments`.`folder` = ". $folder->getID();
 		$queryStr .= " LIMIT 1";
@@ -401,160 +423,308 @@ class LetoDMS_Core_DMS {
 	 * @param query string seach query with space separated words
 	 * @param limit integer number of items in result set
 	 * @param offset integer index of first item in result set
-	 * @param mode string either AND or OR
+	 * @param logicalmode string either AND or OR
 	 * @param searchin array() list of fields to search in
+	 *        1 = keywords, 2=name, 3=comment
 	 * @param startFolder object search in the folder only (null for root folder)
 	 * @param owner object search for documents owned by this user
 	 * @param status array list of status
 	 * @param creationstartdate array search for documents created after this date
 	 * @param creationenddate array search for documents created before this date
+	 * @param categories array list of categories the documents must have assigned
+	 * @param mode int decide whether to search for documents/folders
+	 *        0x1 = documents only
+	 *        0x2 = folders only
+	 *        0x3 = both
 	 * @return array containing the elements total and docs
 	 */
-	function search($query, $limit=0, $offset=0, $mode='AND', $searchin=array(), $startFolder=null, $owner=null, $status = array(), $creationstartdate=array(), $creationenddate=array()) { /* {{{ */
+	function search($query, $limit=0, $offset=0, $logicalmode='AND', $searchin=array(), $startFolder=null, $owner=null, $status = array(), $creationstartdate=array(), $creationenddate=array(), $categories=array(), $mode=0x3) { /* {{{ */
 		// Split the search string into constituent keywords.
 		$tkeys=array();
 		if (strlen($query)>0) {
-			$tkeys = split("[\t\r\n ,]+", $query);
+			$tkeys = preg_split("/[\t\r\n ,]+/", $query);
 		}
 
 		// if none is checkd search all
 		if (count($searchin)==0)
 			$searchin=array( 0, 1, 2, 3);
 
-		$searchKey = "";
-		// Assemble the arguments for the concatenation function. This allows the
-		// search to be carried across all the relevant fields.
-		$concatFunction = "";
-		if (in_array(1, $searchin)) {
-			$concatFunction = "`tblDocuments`.`keywords`";
-		}
-		if (in_array(2, $searchin)) {
-			$concatFunction = (strlen($concatFunction) == 0 ? "" : $concatFunction.", ")."`tblDocuments`.`name`";
-		}
-		if (in_array(3, $searchin)) {
-			$concatFunction = (strlen($concatFunction) == 0 ? "" : $concatFunction.", ")."`tblDocuments`.`comment`";
-		}
+		/*--------- Do it all over again for folders -------------*/
+		if($mode & 0x2) {
+			$searchKey = "";
+			// Assemble the arguments for the concatenation function. This allows the
+			// search to be carried across all the relevant fields.
+			$concatFunction = "";
+			if (in_array(2, $searchin)) {
+				$concatFunction = (strlen($concatFunction) == 0 ? "" : $concatFunction.", ")."`tblFolders`.`name`";
+			}
+			if (in_array(3, $searchin)) {
+				$concatFunction = (strlen($concatFunction) == 0 ? "" : $concatFunction.", ")."`tblFolders`.`comment`";
+			}
 
-		if (strlen($concatFunction)>0 && count($tkeys)>0) {
-			$concatFunction = "CONCAT_WS(' ', ".$concatFunction.")";
-			foreach ($tkeys as $key) {
-				$key = trim($key);
-				if (strlen($key)>0) {
-					$searchKey = (strlen($searchKey)==0 ? "" : $searchKey." ".$mode." ").$concatFunction." LIKE '%".$key."%'";
+			if (strlen($concatFunction)>0 && count($tkeys)>0) {
+				$concatFunction = "CONCAT_WS(' ', ".$concatFunction.")";
+				foreach ($tkeys as $key) {
+					$key = trim($key);
+					if (strlen($key)>0) {
+						$searchKey = (strlen($searchKey)==0 ? "" : $searchKey." ".$logicalmode." ").$concatFunction." LIKE ".$this->db->qstr('%'.$key.'%');
+					}
 				}
 			}
-		}
 
-		// Check to see if the search has been restricted to a particular sub-tree in
-		// the folder hierarchy.
-		$searchFolder = "";
-		if ($startFolder) {
-			$searchFolder = "`tblDocuments`.`folderList` LIKE '%:".$startFolder->getID().":%'";
-		}
-
-		// Check to see if the search has been restricted to a particular
-		// document owner.
-		$searchOwner = "";
-		if ($owner) {
-			$searchOwner = "`tblDocuments`.`owner` = '".$owner->getId()."'";
-		}
-
-		// Is the search restricted to documents created between two specific dates?
-		$searchCreateDate = "";
-		if ($creationstartdate) {
-			$startdate = makeTimeStamp(0, 0, 0, $creationstartdate["year"], $creationstartdate["month"], $creationstartdate["day"]);
-			if ($startdate) {
-				$searchCreateDate .= "`tblDocuments`.`date` >= ".$startdate;
+			// Check to see if the search has been restricted to a particular sub-tree in
+			// the folder hierarchy.
+			$searchFolder = "";
+			if ($startFolder) {
+				$searchFolder = "`tblFolders`.`folderList` LIKE '%:".$startFolder->getID().":%'";
 			}
-		}
-		if ($creationenddate) {
-			$stopdate = makeTimeStamp(23, 59, 59, $creationenddate["year"], $creationenddate["month"], $creationenddate["day"]);
-			if ($stopdate) {
-				if($startdate)
-					$searchCreateDate .= " AND ";
-				$searchCreateDate .= "`tblDocuments`.`date` <= ".$stopdate;
+
+			// Check to see if the search has been restricted to a particular
+			// document owner.
+			$searchOwner = "";
+			if ($owner) {
+				$searchOwner = "`tblFolders`.`owner` = '".$owner->getId()."'";
 			}
+
+			// Is the search restricted to documents created between two specific dates?
+			$searchCreateDate = "";
+			if ($creationstartdate) {
+				$startdate = makeTimeStamp(0, 0, 0, $creationstartdate['year'], $creationstartdate["month"], $creationstartdate["day"]);
+				if ($startdate) {
+					$searchCreateDate .= "`tblFolders`.`date` >= ".$startdate;
+				}
+			}
+			if ($creationenddate) {
+				$stopdate = makeTimeStamp(23, 59, 59, $creationenddate["year"], $creationenddate["month"], $creationenddate["day"]);
+				if ($stopdate) {
+					if($startdate)
+						$searchCreateDate .= " AND ";
+					$searchCreateDate .= "`tblFolders`.`date` <= ".$stopdate;
+				}
+			}
+
+			$searchQuery = "FROM `tblFolders` WHERE 1=1";
+
+			if (strlen($searchKey)>0) {
+				$searchQuery .= " AND (".$searchKey.")";
+			}
+			if (strlen($searchFolder)>0) {
+				$searchQuery .= " AND ".$searchFolder;
+			}
+			if (strlen($searchOwner)>0) {
+				$searchQuery .= " AND (".$searchOwner.")";
+			}
+			if (strlen($searchCreateDate)>0) {
+				$searchQuery .= " AND (".$searchCreateDate.")";
+			}
+
+			// Count the number of rows that the search will produce.
+			$resArr = $this->db->getResultArray("SELECT COUNT(*) AS num ".$searchQuery);
+			$totalFolders = 0;
+			if (is_numeric($resArr[0]["num"]) && $resArr[0]["num"]>0) {
+				$totalFolders = (integer)$resArr[0]["num"];
+			}
+
+			// If there are no results from the count query, then there is no real need
+			// to run the full query. TODO: re-structure code to by-pass additional
+			// queries when no initial results are found.
+
+			// Only search if the offset is not beyond the number of folders
+			if($totalFolders > $offset) {
+				// Prepare the complete search query, including the LIMIT clause.
+				$searchQuery = "SELECT `tblFolders`.* ".$searchQuery;
+
+				if($limit) {
+					$searchQuery .= " LIMIT ".$offset.",".$limit;
+				}
+
+				// Send the complete search query to the database.
+				$resArr = $this->db->getResultArray($searchQuery);
+			} else {
+				$resArr = array();
+			}
+
+			// ------------------- Ausgabe der Ergebnisse ----------------------------
+			$numResults = count($resArr);
+			if ($numResults == 0) {
+				$folderresult = array('totalFolders'=>$totalFolders, 'folders'=>array());
+			} else {
+				foreach ($resArr as $folderArr) {
+					$folders[] = $this->getFolder($folderArr['id']);
+				}
+				$folderresult = array('totalFolders'=>$totalFolders, 'folders'=>$folders);
+			}
+		} else {
+			$folderresult = array('totalFolders'=>0, 'folders'=>array());
 		}
 
-		// ---------------------- Suche starten ----------------------------------
+		/*--------- Do it all over again for documents -------------*/
 
-		//
-		// Construct the SQL query that will be used to search the database.
-		//
+		if($mode & 0x1) {
+			$searchKey = "";
+			// Assemble the arguments for the concatenation function. This allows the
+			// search to be carried across all the relevant fields.
+			$concatFunction = "";
+			if (in_array(1, $searchin)) {
+				$concatFunction = "`tblDocuments`.`keywords`";
+			}
+			if (in_array(2, $searchin)) {
+				$concatFunction = (strlen($concatFunction) == 0 ? "" : $concatFunction.", ")."`tblDocuments`.`name`";
+			}
+			if (in_array(3, $searchin)) {
+				$concatFunction = (strlen($concatFunction) == 0 ? "" : $concatFunction.", ")."`tblDocuments`.`comment`";
+			}
 
-		if (!$this->db->createTemporaryTable("ttcontentid") || !$this->db->createTemporaryTable("ttstatid")) {
-			return false;
+			if (strlen($concatFunction)>0 && count($tkeys)>0) {
+				$concatFunction = "CONCAT_WS(' ', ".$concatFunction.")";
+				foreach ($tkeys as $key) {
+					$key = trim($key);
+					if (strlen($key)>0) {
+						$searchKey = (strlen($searchKey)==0 ? "" : $searchKey." ".$logicalmode." ").$concatFunction." LIKE ".$this->db->qstr('%'.$key.'%');
+					}
+				}
+			}
+
+			// Check to see if the search has been restricted to a particular sub-tree in
+			// the folder hierarchy.
+			$searchFolder = "";
+			if ($startFolder) {
+				$searchFolder = "`tblDocuments`.`folderList` LIKE '%:".$startFolder->getID().":%'";
+			}
+
+			// Check to see if the search has been restricted to a particular
+			// document owner.
+			$searchOwner = "";
+			if ($owner) {
+				$searchOwner = "`tblDocuments`.`owner` = '".$owner->getId()."'";
+			}
+
+			// Check to see if the search has been restricted to a particular
+			// document category.
+			$searchCategories = "";
+			if ($categories) {
+				$catids = array();
+				foreach($categories as $category)
+					$catids[] = $category->getId();
+				$searchCategories = "`tblDocumentCategory`.`categoryID` in (".implode(',', $catids).")";
+			}
+
+			// Is the search restricted to documents created between two specific dates?
+			$searchCreateDate = "";
+			if ($creationstartdate) {
+				$startdate = makeTimeStamp(0, 0, 0, $creationstartdate['year'], $creationstartdate["month"], $creationstartdate["day"]);
+				if ($startdate) {
+					$searchCreateDate .= "`tblDocuments`.`date` >= ".$startdate;
+				}
+			}
+			if ($creationenddate) {
+				$stopdate = makeTimeStamp(23, 59, 59, $creationenddate["year"], $creationenddate["month"], $creationenddate["day"]);
+				if ($stopdate) {
+					if($startdate)
+						$searchCreateDate .= " AND ";
+					$searchCreateDate .= "`tblDocuments`.`date` <= ".$stopdate;
+				}
+			}
+
+			// ---------------------- Suche starten ----------------------------------
+
+			//
+			// Construct the SQL query that will be used to search the database.
+			//
+
+			if (!$this->db->createTemporaryTable("ttcontentid") || !$this->db->createTemporaryTable("ttstatid")) {
+				return false;
+			}
+
+			$searchQuery = "FROM `tblDocumentContent` ".
+				"LEFT JOIN `tblDocuments` ON `tblDocuments`.`id` = `tblDocumentContent`.`document` ".
+				"LEFT JOIN `tblDocumentStatus` ON `tblDocumentStatus`.`documentID` = `tblDocumentContent`.`document` ".
+				"LEFT JOIN `tblDocumentStatusLog` ON `tblDocumentStatusLog`.`statusID` = `tblDocumentStatus`.`statusID` ".
+				"LEFT JOIN `ttstatid` ON `ttstatid`.`maxLogID` = `tblDocumentStatusLog`.`statusLogID` ".
+				"LEFT JOIN `ttcontentid` ON `ttcontentid`.`maxVersion` = `tblDocumentStatus`.`version` AND `ttcontentid`.`document` = `tblDocumentStatus`.`documentID` ".
+				"LEFT JOIN `tblDocumentLocks` ON `tblDocuments`.`id`=`tblDocumentLocks`.`document` ".
+				"LEFT JOIN `tblDocumentCategory` ON `tblDocuments`.`id`=`tblDocumentCategory`.`documentID` ".
+				"WHERE `ttstatid`.`maxLogID`=`tblDocumentStatusLog`.`statusLogID` ".
+				"AND `ttcontentid`.`maxVersion` = `tblDocumentContent`.`version`";
+
+			if (strlen($searchKey)>0) {
+				$searchQuery .= " AND (".$searchKey.")";
+			}
+			if (strlen($searchFolder)>0) {
+				$searchQuery .= " AND ".$searchFolder;
+			}
+			if (strlen($searchOwner)>0) {
+				$searchQuery .= " AND (".$searchOwner.")";
+			}
+			if (strlen($searchCategories)>0) {
+				$searchQuery .= " AND (".$searchCategories.")";
+			}
+			if (strlen($searchCreateDate)>0) {
+				$searchQuery .= " AND (".$searchCreateDate.")";
+			}
+
+			// status
+			if ($status) {
+				$searchQuery .= " AND `tblDocumentStatusLog`.`status` IN (".implode(',', $status).")";
+			}
+
+			// Count the number of rows that the search will produce.
+			$resArr = $this->db->getResultArray("SELECT COUNT(*) AS num ".$searchQuery);
+			$totalDocs = 0;
+			if (is_numeric($resArr[0]["num"]) && $resArr[0]["num"]>0) {
+				$totalDocs = (integer)$resArr[0]["num"];
+			}
+
+			// If there are no results from the count query, then there is no real need
+			// to run the full query. TODO: re-structure code to by-pass additional
+			// queries when no initial results are found.
+
+			// Prepare the complete search query, including the LIMIT clause.
+			$searchQuery = "SELECT `tblDocuments`.*, ".
+				"`tblDocumentContent`.`version`, ".
+				"`tblDocumentStatusLog`.`status`, `tblDocumentLocks`.`userID` as `lockUser` ".$searchQuery;
+
+			// calculate the remaining entrїes of the current page
+			// If page is not full yet, get remaining entries
+			$remain = $limit - count($folderresult['folders']);
+			if($remain) {
+				if($remain == $limit)
+					$offset -= $totalFolders;
+				else
+					$offset = 0;
+				if($limit)
+					$searchQuery .= " LIMIT ".$offset.",".$remain;
+
+				// Send the complete search query to the database.
+				$resArr = $this->db->getResultArray($searchQuery);
+			} else {
+				$resArr = array();
+			}
+
+			// ------------------- Ausgabe der Ergebnisse ----------------------------
+			$numResults = count($resArr);
+			if ($numResults == 0) {
+				$docresult = array('totalDocs'=>$totalDocs, 'docs'=>array());
+			} else {
+				foreach ($resArr as $docArr) {
+					$docs[] = $this->getDocument($docArr['id']);
+				}
+				$docresult = array('totalDocs'=>$totalDocs, 'docs'=>$docs);
+			}
+		} else {
+			$docresult = array('totalDocs'=>0, 'docs'=>array());
 		}
 
-		$searchQuery = "FROM `tblDocumentContent` ".
-			"LEFT JOIN `tblDocuments` ON `tblDocuments`.`id` = `tblDocumentContent`.`document` ".
-			"LEFT JOIN `tblDocumentStatus` ON `tblDocumentStatus`.`documentID` = `tblDocumentContent`.`document` ".
-			"LEFT JOIN `tblDocumentStatusLog` ON `tblDocumentStatusLog`.`statusID` = `tblDocumentStatus`.`statusID` ".
-			"LEFT JOIN `ttstatid` ON `ttstatid`.`maxLogID` = `tblDocumentStatusLog`.`statusLogID` ".
-			"LEFT JOIN `ttcontentid` ON `ttcontentid`.`maxVersion` = `tblDocumentStatus`.`version` AND `ttcontentid`.`document` = `tblDocumentStatus`.`documentID` ".
-			"LEFT JOIN `tblDocumentLocks` ON `tblDocuments`.`id`=`tblDocumentLocks`.`document` ".
-			"WHERE `ttstatid`.`maxLogID`=`tblDocumentStatusLog`.`statusLogID` ".
-			"AND `ttcontentid`.`maxVersion` = `tblDocumentContent`.`version`";
-
-		if (strlen($searchKey)>0) {
-			$searchQuery .= " AND (".$searchKey.")";
-		}
-		if (strlen($searchFolder)>0) {
-			$searchQuery .= " AND ".$searchFolder;
-		}
-		if (strlen($searchOwner)>0) {
-			$searchQuery .= " AND (".$searchOwner.")";
-		}
-		if (strlen($searchCreateDate)>0) {
-			$searchQuery .= " AND (".$searchCreateDate.")";
-		}
-
-		// status
-		if ($status) {
-			$searchQuery .= " AND `tblDocumentStatusLog`.`status` IN (".implode(',', $status).")";
-		}
-
-		// Count the number of rows that the search will produce.
-		$resArr = $this->db->getResultArray("SELECT COUNT(*) AS num ".$searchQuery);
-		$totalDocs = 0;
-		if (is_numeric($resArr[0]["num"]) && $resArr[0]["num"]>0) {
-			$totalDocs = (integer)$resArr[0]["num"];
-		}
 		if($limit) {
-			$totalPages = (integer)($totalDocs/$limit);
-			if (($totalDocs%$limit) > 0) {
+			$totalPages = (integer)(($totalDocs+$totalFolders)/$limit);
+			if ((($totalDocs+$totalFolders)%$limit) > 0) {
 				$totalPages++;
 			}
 		} else {
 			$totalPages = 1;
 		}
 
-		// If there are no results from the count query, then there is no real need
-		// to run the full query. TODO: re-structure code to by-pass additional
-		// queries when no initial results are found.
-
-		// Prepare the complete search query, including the LIMIT clause.
-		$searchQuery = "SELECT `tblDocuments`.*, ".
-			"`tblDocumentContent`.`version`, ".
-			"`tblDocumentStatusLog`.`status`, `tblDocumentLocks`.`userID` as `lockUser` ".$searchQuery;
-
-		if($limit) {
-			$searchQuery .= " LIMIT ".$offset.",".$limit;
-		}
-
-		// Send the complete search query to the database.
-		$resArr = $this->db->getResultArray($searchQuery);
-
-		// ------------------- Ausgabe der Ergebnisse ----------------------------
-		$numResults = count($resArr);
-		if ($numResults == 0) {
-			return array('totalDocs'=>$totalDocs, 'totalPages'=>$totalPages, 'docs'=>array());
-		}
-
-		foreach ($resArr as $docArr) {
-			$docs[] = $this->getDocument($docArr['id']);
-		}
-		return(array('totalDocs'=>$totalDocs, 'totalPages'=>$totalPages, 'docs'=>$docs));
+		return array_merge($docresult, $folderresult, array('totalPages'=>$totalPages));
 	} /* }}} */
 
 	/**
@@ -568,7 +738,7 @@ class LetoDMS_Core_DMS {
 	function getFolder($id) { /* {{{ */
 		if (!is_numeric($id)) return false;
 
-		$queryStr = "SELECT * FROM tblFolders WHERE id = " . $id;
+		$queryStr = "SELECT * FROM tblFolders WHERE id = " . (int) $id;
 		$resArr = $this->db->getResultArray($queryStr);
 
 		if (is_bool($resArr) && $resArr == false)
@@ -597,7 +767,7 @@ class LetoDMS_Core_DMS {
 	function getFolderByName($name, $folder=null) { /* {{{ */
 		if (!$name) return false;
 
-		$queryStr = "SELECT * FROM tblFolders WHERE name = '" . $name . "'";
+		$queryStr = "SELECT * FROM tblFolders WHERE name = " . $this->db->qstr($name);
 		if($folder)
 			$queryStr .= " AND `parent` = ". $folder->getID();
 		$queryStr .= " LIMIT 1";
@@ -627,7 +797,7 @@ class LetoDMS_Core_DMS {
 		if (!is_numeric($id))
 			return false;
 
-		$queryStr = "SELECT * FROM tblUsers WHERE id = " . $id;
+		$queryStr = "SELECT * FROM tblUsers WHERE id = " . (int) $id;
 		$resArr = $this->db->getResultArray($queryStr);
 
 		if (is_bool($resArr) && $resArr == false) return false;
@@ -644,12 +814,40 @@ class LetoDMS_Core_DMS {
 	 * Return a user by its login
 	 *
 	 * This function retrieves a user from the database by its login.
+	 * If the second optional parameter $email is not empty, the user must
+	 * also have the given email.
 	 *
-	 * @param integer $login internal login of user
+	 * @param string $login internal login of user
+	 * @param string $email email of user
 	 * @return object instance of LetoDMS_Core_User or false
 	 */
-	function getUserByLogin($login) { /* {{{ */
-		$queryStr = "SELECT * FROM tblUsers WHERE login = '".$login."'";
+	function getUserByLogin($login, $email='') { /* {{{ */
+		$queryStr = "SELECT * FROM tblUsers WHERE login = ".$this->db->qstr($login);
+		if($email)
+			$queryStr .= " AND email=".$this->db->qstr($email);
+		$resArr = $this->db->getResultArray($queryStr);
+
+		if (is_bool($resArr) && $resArr == false) return false;
+		if (count($resArr) != 1) return false;
+
+		$resArr = $resArr[0];
+
+		$user = new LetoDMS_Core_User($resArr["id"], $resArr["login"], $resArr["pwd"], $resArr["fullName"], $resArr["email"], $resArr["language"], $resArr["theme"], $resArr["comment"], $resArr["role"], $resArr["hidden"]);
+		$user->setDMS($this);
+		return $user;
+	} /* }}} */
+
+	/**
+	 * Return a user by its email
+	 *
+	 * This function retrieves a user from the database by its email.
+	 * It is needed when the user requests a new password.
+	 *
+	 * @param integer $email email address of user
+	 * @return object instance of LetoDMS_Core_User or false
+	 */
+	function getUserByEmail($email) { /* {{{ */
+		$queryStr = "SELECT * FROM tblUsers WHERE email = ".$this->db->qstr($email);
 		$resArr = $this->db->getResultArray($queryStr);
 
 		if (is_bool($resArr) && $resArr == false) return false;
@@ -698,11 +896,14 @@ class LetoDMS_Core_DMS {
 	 *        is still allowed
 	 * @return object of LetoDMS_Core_User
 	 */
-	function addUser($login, $pwd, $fullName, $email, $language, $theme, $comment, $role=0, $isHidden=0) { /* {{{ */
+	function addUser($login, $pwd, $fullName, $email, $language, $theme, $comment, $role='0', $isHidden=0) { /* {{{ */
+		$db = $this->db;
 		if (is_object($this->getUserByLogin($login))) {
 			return false;
 		}
-		$queryStr = "INSERT INTO tblUsers (login, pwd, fullName, email, language, theme, comment, role, hidden) VALUES ('".$login."', '".$pwd."', '".$fullName."', '".$email."', '".$language."', '".$theme."', '".$comment."', '".$role."', '".$isHidden."')";
+		if($role == '')
+			$role = '0';
+		$queryStr = "INSERT INTO tblUsers (login, pwd, fullName, email, language, theme, comment, role, hidden) VALUES (".$db->qstr($login).", ".$db->qstr($pwd).", ".$db->qstr($fullName).", ".$db->qstr($email).", '".$language."', '".$theme."', ".$db->qstr($comment).", '".intval($role)."', '".intval($isHidden)."')";
 		$res = $this->db->getResult($queryStr);
 		if (!$res)
 			return false;
@@ -720,7 +921,7 @@ class LetoDMS_Core_DMS {
 		if (!is_numeric($id))
 			return false;
 
-		$queryStr = "SELECT * FROM tblGroups WHERE id = " . $id;
+		$queryStr = "SELECT * FROM tblGroups WHERE id = " . (int) $id;
 		$resArr = $this->db->getResultArray($queryStr);
 
 		if (is_bool($resArr) && $resArr == false)
@@ -742,7 +943,7 @@ class LetoDMS_Core_DMS {
 	 * @return object/boolean group or false if no group was found
 	 */
 	function getGroupByName($name) { /* {{{ */
-		$queryStr = "SELECT `tblGroups`.* FROM `tblGroups` WHERE `tblGroups`.`name` = '".$name."'";
+		$queryStr = "SELECT `tblGroups`.* FROM `tblGroups` WHERE `tblGroups`.`name` = ".$this->db->qstr($name);
 		$resArr = $this->db->getResultArray($queryStr);
 
 		if (is_bool($resArr) && $resArr == false)
@@ -794,7 +995,7 @@ class LetoDMS_Core_DMS {
 			return false;
 		}
 
-		$queryStr = "INSERT INTO tblGroups (name, comment) VALUES ('".$name."', '" . $comment . "')";
+		$queryStr = "INSERT INTO tblGroups (name, comment) VALUES (".$this->db->qstr($name).", ".$this->db->qstr($comment).")";
 		if (!$this->db->getResult($queryStr))
 			return false;
 
@@ -805,7 +1006,7 @@ class LetoDMS_Core_DMS {
 		if (!is_numeric($id))
 			return false;
 
-		$queryStr = "SELECT * FROM tblKeywordCategories WHERE id = " . $id;
+		$queryStr = "SELECT * FROM tblKeywordCategories WHERE id = " . (int) $id;
 		$resArr = $this->db->getResultArray($queryStr);
 		if ((is_bool($resArr) && !$resArr) || (count($resArr) != 1))
 			return false;
@@ -816,8 +1017,8 @@ class LetoDMS_Core_DMS {
 		return $cat;
 	} /* }}} */
 
-	function getKeywordCategoryByName($name, $owner) { /* {{{ */
-		$queryStr = "SELECT * FROM tblKeywordCategories WHERE name = '" . $name . "' AND owner = '" . $owner. "'";
+	function getKeywordCategoryByName($name, $userID) { /* {{{ */
+		$queryStr = "SELECT * FROM tblKeywordCategories WHERE name = " . $this->db->qstr($name) . " AND owner = " . (int) $userID;
 		$resArr = $this->db->getResultArray($queryStr);
 		if ((is_bool($resArr) && !$resArr) || (count($resArr) != 1))
 			return false;
@@ -847,10 +1048,13 @@ class LetoDMS_Core_DMS {
 		return $categories;
 	} /* }}} */
 
+	/**
+	 * This function should be replaced by getAllKeywordCategories()
+	 */
 	function getAllUserKeywordCategories($userID) { /* {{{ */
 		$queryStr = "SELECT * FROM tblKeywordCategories";
 		if ($userID != -1)
-			$queryStr .= " WHERE owner = " . $userID;
+			$queryStr .= " WHERE owner = " . (int) $userID;
 
 		$resArr = $this->db->getResultArray($queryStr);
 		if (is_bool($resArr) && !$resArr)
@@ -866,17 +1070,82 @@ class LetoDMS_Core_DMS {
 		return $categories;
 	} /* }}} */
 
-	function addKeywordCategory($owner, $name) { /* {{{ */
-		if (is_object($this->getKeywordCategoryByName($name, $owner))) {
+	function addKeywordCategory($userID, $name) { /* {{{ */
+		if (is_object($this->getKeywordCategoryByName($name, $userID))) {
 			return false;
 		}
-		$queryStr = "INSERT INTO tblKeywordCategories (owner, name) VALUES ($owner, '$name')";
+		$queryStr = "INSERT INTO tblKeywordCategories (owner, name) VALUES (".(int) $userID.", ".$this->db->qstr($name).")";
 		if (!$this->db->getResult($queryStr))
 			return false;
 
 		return $this->getKeywordCategory($this->db->getInsertID());
 	} /* }}} */
 
+	function getDocumentCategory($id) { /* {{{ */
+		if (!is_numeric($id))
+			return false;
+
+		$queryStr = "SELECT * FROM tblCategory WHERE id = " . (int) $id;
+		$resArr = $this->db->getResultArray($queryStr);
+		if ((is_bool($resArr) && !$resArr) || (count($resArr) != 1))
+			return false;
+
+		$resArr = $resArr[0];
+		$cat = new LetoDMS_Core_DocumentCategory($resArr["id"], $resArr["name"]);
+		$cat->setDMS($this);
+		return $cat;
+	} /* }}} */
+
+	function getDocumentCategories() { /* {{{ */
+		$queryStr = "SELECT * FROM tblCategory";
+
+		$resArr = $this->db->getResultArray($queryStr);
+		if (is_bool($resArr) && !$resArr)
+			return false;
+
+		$categories = array();
+		foreach ($resArr as $row) {
+			$cat = new LetoDMS_Core_DocumentCategory($row["id"], $row["name"]);
+			$cat->setDMS($this);
+			array_push($categories, $cat);
+		}
+
+		return $categories;
+	} /* }}} */
+
+	/**
+	 * Get a category by its name
+	 *
+	 * The name of a category is by default unique.
+	 *
+	 * @param string $name human readable name of category
+	 * @return object instance of LetoDMS_Core_DocumentCategory
+	 */
+	function getDocumentCategoryByName($name) { /* {{{ */
+		$queryStr = "SELECT * FROM tblCategory where name=".$this->db->qstr($name);
+
+		$resArr = $this->db->getResultArray($queryStr);
+		if (!$resArr)
+			return false;
+
+		$row = $resArr[0];
+		$cat = new LetoDMS_Core_DocumentCategory($row["id"], $row["name"]);
+		$cat->setDMS($this);
+
+		return $cat;
+	} /* }}} */
+
+	function addDocumentCategory($name) { /* {{{ */
+		if (is_object($this->getDocumentCategoryByName($name))) {
+			return false;
+		}
+		$queryStr = "INSERT INTO tblCategory (name) VALUES (".$this->db->qstr($name).")";
+		if (!$this->db->getResult($queryStr))
+			return false;
+
+		return $this->getDocumentCategory($this->db->getInsertID());
+	} /* }}} */
+
 	/**
 	 * Get all notifications for a group
 	 *
@@ -888,7 +1157,7 @@ class LetoDMS_Core_DMS {
 		$queryStr = "SELECT `tblNotify`.* FROM `tblNotify` ".
 		 "WHERE `tblNotify`.`groupID` = ". $group->getID();
 		if($type) {
-			$queryStr .= " AND `tblNotify`.`targetType` = ".$type;
+			$queryStr .= " AND `tblNotify`.`targetType` = ". (int) $type;
 		}
 
 		$resArr = $this->db->getResultArray($queryStr);
@@ -912,11 +1181,11 @@ class LetoDMS_Core_DMS {
 	 * @param integer $type type of item (T_DOCUMENT or T_FOLDER)
 	 * @return array array of notifications
 	 */
-	function getNotificationsByUser($user, $type) { /* {{{ */
+	function getNotificationsByUser($user, $type=0) { /* {{{ */
 		$queryStr = "SELECT `tblNotify`.* FROM `tblNotify` ".
 		 "WHERE `tblNotify`.`userID` = ". $user->getID();
 		if($type) {
-			$queryStr .= " AND `tblNotify`.`targetType` = ".$type;
+			$queryStr .= " AND `tblNotify`.`targetType` = ". (int) $type;
 		}
 
 		$resArr = $this->db->getResultArray($queryStr);
@@ -933,5 +1202,55 @@ class LetoDMS_Core_DMS {
 		return $notifications;
 	} /* }}} */
 
+	/**
+	 * Create a token to request a new password.
+	 * This function will not delete the password but just creates an entry
+	 * in tblUserRequestPassword indicating a password request.
+	 *
+	 * @return string hash value of false in case of an error
+	 */
+	function createPasswordRequest($user) { /* {{{ */
+		$hash = md5(uniqid(time()));
+		$queryStr = "INSERT INTO tblUserPasswordRequest (userID, hash, `date`) VALUES (" . $user->getId() . ", " . $this->db->qstr($hash) .", now())";
+		$resArr = $this->db->getResult($queryStr);
+		if (is_bool($resArr) && !$resArr) return false;
+		return $hash;
+
+	} /* }}} */
+
+	/**
+	 * Check if hash for a password request is valid.
+	 * This function searches a previously create password request and
+	 * returns the user.
+	 *
+	 * @param string $hash
+	 */
+	function checkPasswordRequest($hash) { /* {{{ */
+		/* Get the password request from the database */
+		$queryStr = "SELECT * FROM tblUserPasswordRequest where hash=".$this->db->qstr($hash);
+		$resArr = $this->db->getResultArray($queryStr);
+		if (is_bool($resArr) && !$resArr)
+			return false;
+
+		if (count($resArr) != 1)
+			return false;
+		$resArr = $resArr[0];
+
+		return $this->getUser($resArr['userID']);
+
+	} /* }}} */
+
+	/**
+	 * Delete a password request
+	 *
+	 * @param string $hash
+	 */
+	function deletePasswordRequest($hash) { /* {{{ */
+		/* Delete the request, so nobody can use it a second time */
+		$queryStr = "DELETE FROM tblUserPasswordRequest WHERE hash=".$this->db->qstr($hash);
+		if (!$this->db->getResult($queryStr))
+			return false;
+		return true;
+	}
 }
 ?>
