@@ -9,7 +9,7 @@
  *             Uwe Steinmann <uwe@steinmann.cx>
  * @copyright  Copyright (C) 2002-2005 Markus Westphal, 2006-2008 Malcolm Cowe,
  *             2010 Matteo Lucarelli, 2010 Uwe Steinmann
- * @version    Release: 3.0.1
+ * @version    Release: 3.3.11
  */
 
 /**
@@ -25,7 +25,7 @@
  * @author     Uwe Steinmann <uwe@steinmann.cx>
  * @copyright  Copyright (C) 2002-2005 Markus Westphal, 2006-2008 Malcolm Cowe,
  *             2010 Matteo Lucarelli, 2010 Uwe Steinmann
- * @version    Release: 3.0.1
+ * @version    Release: 3.3.11
  */
 class LetoDMS_Core_Folder {
 	/**
@@ -126,13 +126,13 @@ class LetoDMS_Core_Folder {
 	 */
 	function setName($newName) { /* {{{ */
 		$db = $this->_dms->getDB();
-		
-		$queryStr = "UPDATE tblFolders SET name = '" . $newName . "' WHERE id = ". $this->_id;
+
+		$queryStr = "UPDATE tblFolders SET name = " . $db->qstr($newName) . " WHERE id = ". $this->_id;
 		if (!$db->getResult($queryStr))
 			return false;
 
 		$this->_name = $newName;
-		
+
 		return true;
 	} /* }}} */
 
@@ -140,8 +140,8 @@ class LetoDMS_Core_Folder {
 
 	function setComment($newComment) { /* {{{ */
 		$db = $this->_dms->getDB();
-		
-		$queryStr = "UPDATE tblFolders SET comment = '" . $newComment . "' WHERE id = ". $this->_id;
+
+		$queryStr = "UPDATE tblFolders SET comment = " . $db->qstr($newComment) . " WHERE id = ". $this->_id;
 		if (!$db->getResult($queryStr))
 			return false;
 
@@ -190,7 +190,16 @@ class LetoDMS_Core_Folder {
 			return false;
 		}
 
-		$queryStr = "UPDATE tblFolders SET parent = " . $newParent->getID() . " WHERE id = ". $this->_id;
+		// Update the folderList of the folder
+		$pathPrefix="";
+		$path = $newParent->getPath();
+		foreach ($path as $f) {
+			$pathPrefix .= ":".$f->getID();
+		}
+		if (strlen($pathPrefix)>1) {
+			$pathPrefix .= ":";
+		}
+		$queryStr = "UPDATE tblFolders SET parent = ".$newParent->getID().", folderList='".$pathPrefix."' WHERE id = ". $this->_id;
 		$res = $db->getResult($queryStr);
 		if (!$res)
 			return false;
@@ -214,7 +223,7 @@ class LetoDMS_Core_Folder {
 			return false;
 
 		foreach ($resArr as $row) {
-			$newPath = ereg_replace("^.*:".$this->_id.":(.*$)", $pathPrefix."\\1", $row["folderList"]);
+			$newPath = preg_replace("/^.*:".$this->_id.":(.*$)/", $pathPrefix."\\1", $row["folderList"]);
 			$queryStr="UPDATE `tblDocuments` SET `folderList` = '".$newPath."' WHERE `tblDocuments`.`id` = '".$row["id"]."'";
 			$res = $db->getResult($queryStr);
 		}
@@ -257,14 +266,14 @@ class LetoDMS_Core_Folder {
 			if (!$res) return false;
 			return $this->_parent->getDefaultAccess();
 		}
-		
+
 		return $this->_defaultAccess;
 	} /* }}} */
 
 	function setDefaultAccess($mode) { /* {{{ */
 		$db = $this->_dms->getDB();
 
-		$queryStr = "UPDATE tblFolders set defaultAccess = " . $mode . " WHERE id = " . $this->_id;
+		$queryStr = "UPDATE tblFolders set defaultAccess = " . (int) $mode . " WHERE id = " . $this->_id;
 		if (!$db->getResult($queryStr))
 			return false;
 
@@ -295,7 +304,7 @@ class LetoDMS_Core_Folder {
 
 		$inheritAccess = ($inheritAccess) ? "1" : "0";
 
-		$queryStr = "UPDATE tblFolders SET inheritAccess = " . $inheritAccess . " WHERE id = " . $this->_id;
+		$queryStr = "UPDATE tblFolders SET inheritAccess = " . (int) $inheritAccess . " WHERE id = " . $this->_id;
 		if (!$db->getResult($queryStr))
 			return false;
 
@@ -323,11 +332,11 @@ class LetoDMS_Core_Folder {
 
 	function setSequence($seq) { /* {{{ */
 		$db = $this->_dms->getDB();
-		
+
 		$queryStr = "UPDATE tblFolders SET sequence = " . $seq . " WHERE id = " . $this->_id;
 		if (!$db->getResult($queryStr))
 			return false;
-		
+
 		$this->_sequence = $seq;
 		return true;
 	} /* }}} */
@@ -344,29 +353,38 @@ class LetoDMS_Core_Folder {
 	 */
 	function getSubFolders($orderby="") { /* {{{ */
 		$db = $this->_dms->getDB();
-		
+
 		if (!isset($this->_subFolders)) {
 			if ($orderby=="n") $queryStr = "SELECT * FROM tblFolders WHERE parent = " . $this->_id . " ORDER BY name";
 			else $queryStr = "SELECT * FROM tblFolders WHERE parent = " . $this->_id . " ORDER BY sequence";
 			$resArr = $db->getResultArray($queryStr);
 			if (is_bool($resArr) && $resArr == false)
 				return false;
-			
+
 			$this->_subFolders = array();
 			for ($i = 0; $i < count($resArr); $i++)
 //				$this->_subFolders[$i] = new LetoDMS_Core_Folder($resArr[$i]["id"], $resArr[$i]["name"], $resArr[$i]["parent"], $resArr[$i]["comment"], $resArr[$i]["owner"], $resArr[$i]["inheritAccess"], $resArr[$i]["defaultAccess"], $resArr[$i]["sequence"]);
 				$this->_subFolders[$i] = $this->_dms->getFolder($resArr[$i]["id"]);
 		}
-		
+
 		return $this->_subFolders;
 	} /* }}} */
 
 	function addSubFolder($name, $comment, $owner, $sequence) { /* {{{ */
 		$db = $this->_dms->getDB();
 
+		// Set the folderList of the folder
+		$pathPrefix="";
+		$path = $this->getPath();
+		foreach ($path as $f) {
+			$pathPrefix .= ":".$f->getID();
+		}
+		if (strlen($pathPrefix)>1) {
+			$pathPrefix .= ":";
+		}
 		//inheritAccess = true, defaultAccess = M_READ
-		$queryStr = "INSERT INTO tblFolders (name, parent, comment, date, owner, inheritAccess, defaultAccess, sequence) ".
-					"VALUES ('".$name."', ".$this->_id.", '".$comment."', ".mktime().", ".$owner->getID().", 1, ".M_READ.", ".$sequence.")";
+		$queryStr = "INSERT INTO tblFolders (name, parent, folderList, comment, date, owner, inheritAccess, defaultAccess, sequence) ".
+					"VALUES (".$db->qstr($name).", ".$this->_id.", ".$db->qstr($pathPrefix).", ".$db->qstr($comment).", ".mktime().", ".$owner->getID().", 1, ".M_READ.", ". $sequence.")";
 		if (!$db->getResult($queryStr))
 			return false;
 		$newFolder = $this->_dms->getFolder($db->getInsertID());
@@ -375,7 +393,7 @@ class LetoDMS_Core_Folder {
 		return $newFolder;
 	} /* }}} */
 
-	/*
+	/**
 	 * Returns an array of all parents, grand parent, etc. up to root folder.
 	 * The folder itself is the last element of the array.
 	 *
@@ -388,15 +406,20 @@ class LetoDMS_Core_Folder {
 		else {
 			$res = $this->getParent();
 			if (!$res) return false;
-			
+
 			$path = $this->_parent->getPath();
 			if (!$path) return false;
-			
+
 			array_push($path, $this);
 			return $path;
 		}
 	} /* }}} */
 
+	/**
+	 * Returns a unix file system path
+	 *
+	 * @return string path separated with '/'
+	 */
 	function getFolderPathPlain() { /* {{{ */
 		$path="";
 		$folderPath = $this->getPath();
@@ -420,7 +443,7 @@ class LetoDMS_Core_Folder {
 		elseif (isset($this->_parentID)) {
 			$res = $this->getParent();
 			if (!$res) return false;
-			
+
 			return $this->_parent->isDescendant($folder);
 		} else
 			return false;
@@ -438,7 +461,7 @@ class LetoDMS_Core_Folder {
 	 */
 	function getDocuments($orderby="") { /* {{{ */
 		$db = $this->_dms->getDB();
-		
+
 		if (!isset($this->_documents)) {
 			if ($orderby=="n") $queryStr = "SELECT * FROM tblDocuments WHERE folder = " . $this->_id . " ORDER BY name";
 			else $queryStr = "SELECT * FROM tblDocuments WHERE folder = " . $this->_id . " ORDER BY sequence";
@@ -446,7 +469,7 @@ class LetoDMS_Core_Folder {
 			$resArr = $db->getResultArray($queryStr);
 			if (is_bool($resArr) && !$resArr)
 				return false;
-			
+
 			$this->_documents = array();
 			foreach ($resArr as $row) {
 //				array_push($this->_documents, new LetoDMS_Core_Document($row["id"], $row["name"], $row["comment"], $row["date"], $row["expires"], $row["owner"], $row["folder"], $row["inheritAccess"], $row["defaultAccess"], isset($row["lockUser"])?$row["lockUser"]:NULL, $row["keywords"], $row["sequence"]));
@@ -469,11 +492,12 @@ class LetoDMS_Core_Folder {
 	 *        expiration date
 	 * @param object $owner owner of the new document
 	 * @param string $keywords keywords of new document
+	 * @param array $categories list of category ids
 	 * @param string $tmpFile the path of the file containing the content
 	 * @param string $orgFileName the original file name
 	 * @param string $fileType usually the extension of the filename
 	 * @param string $mimeType mime type of the content
-	 * @param integer $sequence position of new document within the folder
+	 * @param float $sequence position of new document within the folder
 	 * @param array $reviewers list of users who must review this document
 	 * @param array $approvers list of users who must approve this document
 	 * @param string $reqversion version number of the content
@@ -483,11 +507,11 @@ class LetoDMS_Core_Folder {
 	 *        containing two elements. The first one is the new document, the
 	 *        second one is the result set returned when inserting the content.
 	 */
-	function addDocument($name, $comment, $expires, $owner, $keywords, $tmpFile, $orgFileName, $fileType, $mimeType, $sequence, $reviewers=array(), $approvers=array(),$reqversion,$version_comment="") { /* {{{ */
+	function addDocument($name, $comment, $expires, $owner, $keywords, $categories, $tmpFile, $orgFileName, $fileType, $mimeType, $sequence, $reviewers=array(), $approvers=array(),$reqversion,$version_comment="") { /* {{{ */
 		$db = $this->_dms->getDB();
-		
+
 		$expires = (!$expires) ? 0 : $expires;
-		
+
 		// Must also ensure that the document has a valid folderList.
 		$pathPrefix="";
 		$path = $this->getPath();
@@ -497,14 +521,14 @@ class LetoDMS_Core_Folder {
 		if (strlen($pathPrefix)>1) {
 			$pathPrefix .= ":";
 		}
-		
+
 		$queryStr = "INSERT INTO tblDocuments (name, comment, date, expires, owner, folder, folderList, inheritAccess, defaultAccess, locked, keywords, sequence) VALUES ".
-					"('".$name."', '".$comment."', " . mktime().", ".$expires.", ".$owner->getID().", ".$this->_id.",'".$pathPrefix."', 1, ".M_READ.", -1, '".$keywords."', " . $sequence . ")";
+					"(".$db->qstr($name).", ".$db->qstr($comment).", " . mktime().", ".(int) $expires.", ".$owner->getID().", ".$this->_id.",".$db->qstr($pathPrefix).", 1, ".M_READ.", -1, ".$db->qstr($keywords).", " . $sequence . ")";
 		if (!$db->getResult($queryStr))
 			return false;
-		
+
 		$document = $this->_dms->getDocument($db->getInsertID());
-		
+
 		if ($version_comment!="")
 			$res = $document->addContent($version_comment, $owner, $tmpFile, $orgFileName, $fileType, $mimeType, $reviewers, $approvers,$reqversion);
 		else $res = $document->addContent($comment, $owner, $tmpFile, $orgFileName, $fileType, $mimeType, $reviewers, $approvers,$reqversion);
@@ -515,9 +539,12 @@ class LetoDMS_Core_Folder {
 			return false;
 		}
 
+		if($categories) {
+			$document->setCategories($categories);
+		}
 		return array($document, $res);
 	} /* }}} */
-	
+
 	function remove() { /* {{{ */
 		$db = $this->_dms->getDB();
 
@@ -531,17 +558,17 @@ class LetoDMS_Core_Folder {
 		if (is_bool($res) && !$res) return false;
 		$res = $this->getDocuments();
 		if (is_bool($res) && !$res) return false;
-		
+
 		foreach ($this->_subFolders as $subFolder) {
 			$res = $subFolder->remove(FALSE);
 			if (!$res) return false;
 		}
-		
+
 		foreach ($this->_documents as $document) {
 			$res = $document->remove(FALSE);
 			if (!$res) return false;
 		}
-		
+
 		//Entfernen der Datenbankeintrge
 		$queryStr = "DELETE FROM tblFolders WHERE id =  " . $this->_id;
 		if (!$db->getResult($queryStr))
@@ -553,10 +580,28 @@ class LetoDMS_Core_Folder {
 		$queryStr = "DELETE FROM tblNotify WHERE target = ". $this->_id. " AND targetType = " . T_FOLDER;
 		if (!$db->getResult($queryStr))
 			return false;
-		
+
 		return true;
 	} /* }}} */
 
+	/**
+	 * Returns a list of access privileges
+	 *
+	 * If the document inherits the access privileges from the parent folder
+	 * those will be returned.
+	 * $mode and $op can be set to restrict the list of returned access
+	 * privileges. If $mode is set to M_ANY no restriction will apply
+	 * regardless of the value of $op. The returned array contains a list
+	 * of {@link LetoDMS_Core_UserAccess} and
+	 * {@link LetoDMS_Core_GroupAccess} objects. Even if the document
+	 * has no access list the returned array contains the two elements
+	 * 'users' and 'groups' which are than empty. The methode returns false
+	 * if the function fails.
+	 * 
+	 * @param integer $mode access mode (defaults to M_ANY)
+	 * @param integer $op operation (defaults to O_EQ)
+	 * @return array multi dimensional array
+	 */
 	function getAccessList($mode = M_ANY, $op = O_EQ) { /* {{{ */
 		$db = $this->_dms->getDB();
 
@@ -572,7 +617,7 @@ class LetoDMS_Core_Folder {
 			}
 			$modeStr = "";
 			if ($mode!=M_ANY) {
-				$modeStr = " AND mode".$op.$mode;
+				$modeStr = " AND mode".$op.(int)$mode;
 			}
 			$queryStr = "SELECT * FROM tblACLs WHERE targetType = ".T_FOLDER.
 				" AND target = " . $this->_id .	$modeStr . " ORDER BY targetType";
@@ -592,13 +637,18 @@ class LetoDMS_Core_Folder {
 		return $this->_accessList[$mode];
 	} /* }}} */
 
+	/**
+	 * Delete all entries for this folder from the access control list
+	 *
+	 * @return boolean true if operation was successful otherwise false
+	 */
 	function clearAccessList() { /* {{{ */
 		$db = $this->_dms->getDB();
-		
+
 		$queryStr = "DELETE FROM tblACLs WHERE targetType = " . T_FOLDER . " AND target = " . $this->_id;
 		if (!$db->getResult($queryStr))
 			return false;
-		
+
 		unset($this->_accessList);
 		return true;
 	} /* }}} */
@@ -619,7 +669,7 @@ class LetoDMS_Core_Folder {
 		$userOrGroup = ($isUser) ? "userID" : "groupID";
 
 		$queryStr = "INSERT INTO tblACLs (target, targetType, ".$userOrGroup.", mode) VALUES 
-					(".$this->_id.", ".T_FOLDER.", " . $userOrGroupID . ", " .$mode. ")";
+					(".$this->_id.", ".T_FOLDER.", " . (int) $userOrGroupID . ", " .(int) $mode. ")";
 		if (!$db->getResult($queryStr))
 			return false;
 
@@ -648,7 +698,7 @@ class LetoDMS_Core_Folder {
 
 		$userOrGroup = ($isUser) ? "userID" : "groupID";
 
-		$queryStr = "UPDATE tblACLs SET mode = " . $newMode . " WHERE targetType = ".T_FOLDER." AND target = " . $this->_id . " AND " . $userOrGroup . " = " . $userOrGroupID;
+		$queryStr = "UPDATE tblACLs SET mode = " . (int) $newMode . " WHERE targetType = ".T_FOLDER." AND target = " . $this->_id . " AND " . $userOrGroup . " = " . (int) $userOrGroupID;
 		if (!$db->getResult($queryStr))
 			return false;
 
@@ -667,7 +717,7 @@ class LetoDMS_Core_Folder {
 
 		$userOrGroup = ($isUser) ? "userID" : "groupID";
 
-		$queryStr = "DELETE FROM tblACLs WHERE targetType = ".T_FOLDER." AND target = ".$this->_id." AND ".$userOrGroup." = " . $userOrGroupID;
+		$queryStr = "DELETE FROM tblACLs WHERE targetType = ".T_FOLDER." AND target = ".$this->_id." AND ".$userOrGroup." = " . (int) $userOrGroupID;
 		if (!$db->getResult($queryStr))
 			return false;
 
@@ -684,6 +734,15 @@ class LetoDMS_Core_Folder {
 
 	/**
 	 * Get the access mode of a user on the folder
+	 *
+	 * This function returns the access mode for a given user. An administrator
+	 * and the owner of the folder has unrestricted access. A guest user has
+	 * read only access or no access if access rights are further limited
+	 * by access control lists. All other users have access rights according
+	 * to the access control lists or the default access. This function will
+	 * recursive check for access rights of parent folders if access rights
+	 * are inherited.
+	 *
 	 * This function returns the access mode for a given user. An administrator
 	 * and the owner of the folder has unrestricted access. A guest user has
 	 * read only access or no access if access rights are further limited
@@ -698,17 +757,17 @@ class LetoDMS_Core_Folder {
 	function getAccessMode($user) { /* {{{ */
 		/* Admins have full access */
 		if ($user->isAdmin()) return M_ALL;
-		
+
 		/* User has full access if he/she is the owner of the document */
 		if ($user->getID() == $this->_ownerID) return M_ALL;
-		
+
 		/* Guest has read access by default, if guest login is allowed at all */
 		if ($user->isGuest()) {
 			$mode = $this->getDefaultAccess();
 			if ($mode >= M_READ) return M_READ;
 			else return M_NONE;
 		}
-		
+
 		/* check ACLs */
 		$accessList = $this->getAccessList();
 		if (!$accessList) return false;
@@ -718,13 +777,18 @@ class LetoDMS_Core_Folder {
 				return $userAccess->getMode();
 			}
 		}
-		$result = $this->getDefaultAccess();
+		/* Get the highest right defined by a group */
+		$result = 0;
 		foreach ($accessList["groups"] as $groupAccess) {
 			if ($user->isMemberOfGroup($groupAccess->getGroup())) {
-				if ($groupAccess->getMode()>$result)
+				if ($groupAccess->getMode() > $result)
 					$result = $groupAccess->getMode();
+//					return $groupAccess->getMode();
 			}
 		}
+		if($result)
+			return $result;
+		$result = $this->getDefaultAccess();
 		return $result;
 	} /* }}} */
 
@@ -851,6 +915,8 @@ class LetoDMS_Core_Folder {
 			}
 		}
 		else {
+			// FIXME: Why not check the access list first and if this returns
+			// not result, then use the default access?
 			// Groups are a little more complex.
 			if ($this->getDefaultAccess() >= M_READ) {
 				// If the default access is at least READ-ONLY, then just make sure
@@ -891,7 +957,7 @@ class LetoDMS_Core_Folder {
 		//
 		$queryStr = "SELECT * FROM `tblNotify` WHERE `tblNotify`.`target` = '".$this->_id."' ".
 			"AND `tblNotify`.`targetType` = '".T_FOLDER."' ".
-			"AND `tblNotify`.`".$userOrGroup."` = '".$userOrGroupID."'";
+			"AND `tblNotify`.`".$userOrGroup."` = '". (int) $userOrGroupID."'";
 		$resArr = $db->getResultArray($queryStr);
 		if (is_bool($resArr)) {
 			return -4;
@@ -900,7 +966,7 @@ class LetoDMS_Core_Folder {
 			return -3;
 		}
 
-		$queryStr = "INSERT INTO tblNotify (target, targetType, " . $userOrGroup . ") VALUES (" . $this->_id . ", " . T_FOLDER . ", " . $userOrGroupID . ")";
+		$queryStr = "INSERT INTO tblNotify (target, targetType, " . $userOrGroup . ") VALUES (" . $this->_id . ", " . T_FOLDER . ", " .  (int) $userOrGroupID . ")";
 		if (!$db->getResult($queryStr))
 			return -4;
 
@@ -924,7 +990,7 @@ class LetoDMS_Core_Folder {
 	 */
 	function removeNotify($userOrGroupID, $isUser) { /* {{{ */
 		$db = $this->_dms->getDB();
-		
+
 		/* Verify that user / group exists. */
 		$obj = ($isUser ? $this->_dms->getUser($userOrGroupID) : $this->_dms->getGroup($userOrGroupID));
 		if (!is_object($obj)) {
@@ -964,7 +1030,7 @@ class LetoDMS_Core_Folder {
 		//
 		$queryStr = "SELECT * FROM `tblNotify` WHERE `tblNotify`.`target` = '".$this->_id."' ".
 			"AND `tblNotify`.`targetType` = '".T_FOLDER."' ".
-			"AND `tblNotify`.`".$userOrGroup."` = '".$userOrGroupID."'";
+			"AND `tblNotify`.`".$userOrGroup."` = '". (int) $userOrGroupID."'";
 		$resArr = $db->getResultArray($queryStr);
 		if (is_bool($resArr)) {
 			return -4;
@@ -973,7 +1039,7 @@ class LetoDMS_Core_Folder {
 			return -3;
 		}
 
-		$queryStr = "DELETE FROM tblNotify WHERE target = " . $this->_id . " AND targetType = " . T_FOLDER . " AND " . $userOrGroup . " = " . $userOrGroupID;
+		$queryStr = "DELETE FROM tblNotify WHERE target = " . $this->_id . " AND targetType = " . T_FOLDER . " AND " . $userOrGroup . " = " .  (int) $userOrGroupID;
 		if (!$db->getResult($queryStr))
 			return -4;
 
@@ -1086,6 +1152,53 @@ class LetoDMS_Core_Folder {
 		}
 		return $this->_approversList;
 	} /* }}} */
+
+	/**
+	 * Get the internally used folderList which stores the ids of folders from
+	 * the root folder to the parent folder.
+	 *
+	 * @return string column separated list of folder ids
+	 */
+	function getFolderList() { /* {{{ */
+		$db = $this->_dms->getDB();
+
+		$queryStr = "SELECT folderList FROM tblFolders where id = ".$this->_id;
+		$resArr = $db->getResultArray($queryStr);
+		if (is_bool($resArr) && !$resArr)
+			return false;
+		return $resArr[0]['folderList'];
+	} /* }}} */
+
+	/**
+	 * Checks the internal data of the folder and repairs it.
+	 * Currently, this function only repairs an incorrect folderList
+	 *
+	 * @return boolean true on success, otherwise false
+	 */
+	function repair() { /* {{{ */
+		$db = $this->_dms->getDB();
+
+		$curfolderlist = $this->getFolderList();
+
+		// calculate the folderList of the folder
+		$parent = $this->getParent();
+		$pathPrefix="";
+		$path = $parent->getPath();
+		foreach ($path as $f) {
+			$pathPrefix .= ":".$f->getID();
+		}
+		if (strlen($pathPrefix)>1) {
+			$pathPrefix .= ":";
+		}
+		if($curfolderlist != $pathPrefix) {
+			$queryStr = "UPDATE tblFolders SET folderList='".$pathPrefix."' WHERE id = ". $this->_id;
+			$res = $db->getResult($queryStr);
+			if (!$res)
+				return false;
+		}
+		return true;
+	} /* }}} */
+
 }
 
 ?>
