@@ -9,7 +9,7 @@
  * @author     Uwe Steinmann <uwe@steinmann.cx>
  * @copyright  Copyright (C) 2002-2005 Markus Westphal, 2006-2008 Malcolm Cowe,
  *             2010 Uwe Steinmann
- * @version    Release: 3.0.1
+ * @version    Release: 3.3.11
  */
 
 /**
@@ -20,7 +20,7 @@
  * @author     Markus Westphal, Malcolm Cowe, Uwe Steinmann <uwe@steinmann.cx>
  * @copyright  Copyright (C) 2002-2005 Markus Westphal, 2006-2008 Malcolm Cowe,
  *             2010 Uwe Steinmann
- * @version    Release: 3.0.1
+ * @version    Release: 3.3.11
  */
 class LetoDMS_Core_User {
 	/**
@@ -133,7 +133,7 @@ class LetoDMS_Core_User {
 	function setLogin($newLogin) { /* {{{ */
 		$db = $this->_dms->getDB();
 
-		$queryStr = "UPDATE tblUsers SET login ='" . $newLogin . "' WHERE id = " . $this->_id;
+		$queryStr = "UPDATE tblUsers SET login =".$db->qstr($newLogin)." WHERE id = " . $this->_id;
 		$res = $db->getResult($queryStr);
 		if (!$res)
 			return false;
@@ -147,7 +147,7 @@ class LetoDMS_Core_User {
 	function setFullName($newFullName) { /* {{{ */
 		$db = $this->_dms->getDB();
 
-		$queryStr = "UPDATE tblUsers SET fullname = '" . $newFullName . "' WHERE id = " . $this->_id;
+		$queryStr = "UPDATE tblUsers SET fullname = ".$db->qstr($newFullName)." WHERE id = " . $this->_id;
 		$res = $db->getResult($queryStr);
 		if (!$res)
 			return false;
@@ -161,7 +161,7 @@ class LetoDMS_Core_User {
 	function setPwd($newPwd) { /* {{{ */
 		$db = $this->_dms->getDB();
 
-		$queryStr = "UPDATE tblUsers SET pwd ='" . $newPwd . "' WHERE id = " . $this->_id;
+		$queryStr = "UPDATE tblUsers SET pwd =".$db->qstr($newPwd)." WHERE id = " . $this->_id;
 		$res = $db->getResult($queryStr);
 		if (!$res)
 			return false;
@@ -175,7 +175,7 @@ class LetoDMS_Core_User {
 	function setEmail($newEmail) { /* {{{ */
 		$db = $this->_dms->getDB();
 
-		$queryStr = "UPDATE tblUsers SET email ='" . $newEmail . "' WHERE id = " . $this->_id;
+		$queryStr = "UPDATE tblUsers SET email =".$db->qstr($newEmail)." WHERE id = " . $this->_id;
 		$res = $db->getResult($queryStr);
 		if (!$res)
 			return false;
@@ -189,7 +189,7 @@ class LetoDMS_Core_User {
 	function setLanguage($newLanguage) { /* {{{ */
 		$db = $this->_dms->getDB();
 
-		$queryStr = "UPDATE tblUsers SET language ='" . $newLanguage . "' WHERE id = " . $this->_id;
+		$queryStr = "UPDATE tblUsers SET language =".$db->qstr($newLanguage)." WHERE id = " . $this->_id;
 		$res = $db->getResult($queryStr);
 		if (!$res)
 			return false;
@@ -203,7 +203,7 @@ class LetoDMS_Core_User {
 	function setTheme($newTheme) { /* {{{ */
 		$db = $this->_dms->getDB();
 
-		$queryStr = "UPDATE tblUsers SET theme ='" . $newTheme . "' WHERE id = " . $this->_id;
+		$queryStr = "UPDATE tblUsers SET theme =".$db->qstr($newTheme)." WHERE id = " . $this->_id;
 		$res = $db->getResult($queryStr);
 		if (!$res)
 			return false;
@@ -217,7 +217,7 @@ class LetoDMS_Core_User {
 	function setComment($newComment) { /* {{{ */
 		$db = $this->_dms->getDB();
 
-		$queryStr = "UPDATE tblUsers SET comment ='" . $newComment . "' WHERE id = " . $this->_id;
+		$queryStr = "UPDATE tblUsers SET comment =".$db->qstr($newComment)." WHERE id = " . $this->_id;
 		$res = $db->getResult($queryStr);
 		if (!$res)
 			return false;
@@ -322,7 +322,7 @@ class LetoDMS_Core_User {
 		$queryStr = "DELETE FROM tblNotify WHERE userID = " . $this->_id;
 		if (!$db->getResult($queryStr)) return false;
 
-		//Der Besitz von Dokumenten oder Ordnern, deren bisheriger Besitzer der zu lschende war, geht an den Admin ber
+		/* Assign documents of the removed user to the given user */
 		$queryStr = "UPDATE tblFolders SET owner = " . $assignTo . " WHERE owner = " . $this->_id;
 		if (!$db->getResult($queryStr)) return false;
 
@@ -332,11 +332,11 @@ class LetoDMS_Core_User {
 		$queryStr = "UPDATE tblDocumentContent SET createdBy = " . $assignTo . " WHERE createdBy = " . $this->_id;
 		if (!$db->getResult($queryStr)) return false;
 
-		//Verweise auf Dokumente: Private lschen...
+		// Remove private links on documents ...
 		$queryStr = "DELETE FROM tblDocumentLinks WHERE userID = " . $this->_id . " AND public = 0";
 		if (!$db->getResult($queryStr)) return false;
 
-		//... und ffentliche an Admin bergeben
+		// ... but keep public links
 		$queryStr = "UPDATE tblDocumentLinks SET userID = " . $assignTo . " WHERE userID = " . $this->_id;
 		if (!$db->getResult($queryStr)) return false;
 
@@ -348,19 +348,19 @@ class LetoDMS_Core_User {
 		$queryStr = "DELETE FROM tblDocumentLocks WHERE userID = " . $this->_id;
 		if (!$db->getResult($queryStr)) return false;
 
-		//User aus allen Gruppen lschen
+		// Delete user from all groups
 		$queryStr = "DELETE FROM tblGroupMembers WHERE userID = " . $this->_id;
 		if (!$db->getResult($queryStr)) return false;
 
-		//User aus allen ACLs streichen
+		// User aus allen ACLs streichen
 		$queryStr = "DELETE FROM tblACLs WHERE userID = " . $this->_id;
 		if (!$db->getResult($queryStr)) return false;
 
-		//Eintrag aus tblUserImagess lschen
+		// Delete image of user
 		$queryStr = "DELETE FROM tblUserImages WHERE userID = " . $this->_id;
 		if (!$db->getResult($queryStr)) return false;
 
-		//Eintrag aus tblUsers lschen
+		// Delete user itself
 		$queryStr = "DELETE FROM tblUsers WHERE id = " . $this->_id;
 		if (!$db->getResult($queryStr)) return false;
 
@@ -405,6 +405,15 @@ class LetoDMS_Core_User {
 		return true;
 	} /* }}} */
 
+	/**
+	 * Make the user a member of a group
+	 * This function uses {@link LetoDMS_Group::addUser} but checks before if
+	 * the user is already a member of the group.
+	 *
+	 * @param object $group group to be the member of
+	 * @return boolean true on success or false in case of an error or the user
+	 *        is already a member of the group
+	 */
 	function joinGroup($group) { /* {{{ */
 		if ($group->isMember($this))
 			return false;
@@ -416,6 +425,15 @@ class LetoDMS_Core_User {
 		return true;
 	} /* }}} */
 
+	/**
+	 * Removes the user from a group
+	 * This function uses {@link LetoDMS_Group::removeUser} but checks before if
+	 * the user is a member of the group at all.
+	 *
+	 * @param object $group group to leave
+	 * @return boolean true on success or false in case of an error or the user
+	 *        is not a member of the group
+	 */
 	function leaveGroup($group) { /* {{{ */
 		if (!$group->isMember($this))
 			return false;
@@ -427,6 +445,11 @@ class LetoDMS_Core_User {
 		return true;
 	} /* }}} */
 
+	/**
+	 * Get all groups the user is a member of
+	 *
+	 * @return array list of groups
+	 */
 	function getGroups() { /* {{{ */
 		$db = $this->_dms->getDB();
 
@@ -469,7 +492,7 @@ class LetoDMS_Core_User {
 
 			$queryStr = "SELECT COUNT(*) AS num FROM tblUserImages WHERE userID = " . $this->_id;
 			$resArr = $db->getResultArray($queryStr);
-			if (is_bool($resArr) && $resArr == false)
+			if ($resArr === false)
 				return false;
 
 			if ($resArr[0]["num"] == 0)	$this->_hasImage = false;
@@ -489,10 +512,11 @@ class LetoDMS_Core_User {
 
 		$queryStr = "SELECT * FROM tblUserImages WHERE userID = " . $this->_id;
 		$resArr = $db->getResultArray($queryStr);
-		if (is_bool($resArr) && $resArr == false)
+		if ($resArr === false)
 			return false;
 
-		$resArr = $resArr[0];
+		if($resArr)
+			$resArr = $resArr[0];
 		return $resArr;
 	} /* }}} */
 
@@ -505,9 +529,9 @@ class LetoDMS_Core_User {
 		fclose($fp);
 
 		if ($this->hasImage())
-			$queryStr = "UPDATE tblUserImages SET image = '".base64_encode($content)."', mimeType = '". $mimeType."' WHERE userID = " . $this->_id;
+			$queryStr = "UPDATE tblUserImages SET image = '".base64_encode($content)."', mimeType = ".$db->qstr($mimeType)." WHERE userID = " . $this->_id;
 		else
-			$queryStr = "INSERT INTO tblUserImages (userID, image, mimeType) VALUES (" . $this->_id . ", '".base64_encode($content)."', '".$mimeType."')";
+			$queryStr = "INSERT INTO tblUserImages (userID, image, mimeType) VALUES (" . $this->_id . ", '".base64_encode($content)."', ".$db->qstr($mimeType).")";
 		if (!$db->getResult($queryStr))
 			return false;
 
@@ -515,13 +539,30 @@ class LetoDMS_Core_User {
 		return true;
 	} /* }}} */
 
+	/**
+	 * Get a list of reviews
+	 * This function returns a list of all reviews seperated by individual
+	 * and group reviews. If the document id
+	 * is passed, then only this document will be checked for approvals. The
+	 * same is true for the version of a document which limits the list
+	 * further.
+	 *
+	 * For a detaile description of the result array see
+	 * {link LetoDMS_User::getApprovalStatus}
+	 *
+	 * @param int $documentID optional document id for which to retrieve the
+	 *        reviews
+	 * @param int $version optional version of the document
+	 * @return array list of all reviews
+	 */
 	function getReviewStatus($documentID=null, $version=null) { /* {{{ */
 		$db = $this->_dms->getDB();
 
+/*
 		if (!$db->createTemporaryTable("ttreviewid")) {
 			return false;
 		}
-
+*/
 		$status = array("indstatus"=>array(), "grpstatus"=>array());
 
 		// See if the user is assigned as an individual reviewer.
@@ -530,12 +571,11 @@ class LetoDMS_Core_User {
 			"`tblDocumentReviewLog`.`userID` ".
 			"FROM `tblDocumentReviewers` ".
 			"LEFT JOIN `tblDocumentReviewLog` USING (`reviewID`) ".
-			"LEFT JOIN `ttreviewid` on `ttreviewid`.`maxLogID` = `tblDocumentReviewLog`.`reviewLogID` ".
-			"WHERE `ttreviewid`.`maxLogID`=`tblDocumentReviewLog`.`reviewLogID` ".
-			($documentID==null ? "" : "AND `tblDocumentReviewers`.`documentID` = '". $documentID ."' ").
-			($version==null ? "" : "AND `tblDocumentReviewers`.`version` = '". $version ."' ").
-			"AND `tblDocumentReviewers`.`type`='0' ".
-			"AND `tblDocumentReviewers`.`required`='". $this->_id ."' ";
+			"WHERE `tblDocumentReviewers`.`type`='0' ".
+			($documentID==null ? "" : "AND `tblDocumentReviewers`.`documentID` = '". (int) $documentID ."' ").
+			($version==null ? "" : "AND `tblDocumentReviewers`.`version` = '". (int) $version ."' ").
+			"AND `tblDocumentReviewers`.`required`='". $this->_id ."' ".
+			"ORDER BY `tblDocumentReviewLog`.`reviewLogID` DESC LIMIT 1";
 		$resArr = $db->getResultArray($queryStr);
 		if (is_bool($resArr) && $resArr == false)
 			return false;
@@ -552,12 +592,11 @@ class LetoDMS_Core_User {
 			"FROM `tblDocumentReviewers` ".
 			"LEFT JOIN `tblDocumentReviewLog` USING (`reviewID`) ".
 			"LEFT JOIN `tblGroupMembers` ON `tblGroupMembers`.`groupID` = `tblDocumentReviewers`.`required` ".
-			"LEFT JOIN `ttreviewid` on `ttreviewid`.`maxLogID` = `tblDocumentReviewLog`.`reviewLogID` ".
-			"WHERE `ttreviewid`.`maxLogID`=`tblDocumentReviewLog`.`reviewLogID` ".
-			($documentID==null ? "" : "AND `tblDocumentReviewers`.`documentID` = '". $documentID ."' ").
-			($version==null ? "" : "AND `tblDocumentReviewers`.`version` = '". $version ."' ").
-			"AND `tblDocumentReviewers`.`type`='1' ".
-			"AND `tblGroupMembers`.`userID`='". $this->_id ."'";
+			"WHERE `tblDocumentReviewers`.`type`='1' ".
+			($documentID==null ? "" : "AND `tblDocumentReviewers`.`documentID` = '". (int) $documentID ."' ").
+			($version==null ? "" : "AND `tblDocumentReviewers`.`version` = '". (int) $version ."' ").
+			"AND `tblGroupMembers`.`userID`='". $this->_id ."' ".
+			"ORDER BY `tblDocumentReviewLog`.`reviewLogID` DESC LIMIT 1";
 		$resArr = $db->getResultArray($queryStr);
 		if (is_bool($resArr) && $resArr == false)
 			return false;
@@ -568,16 +607,46 @@ class LetoDMS_Core_User {
 		return $status;
 	} /* }}} */
 
+	/**
+	 * Get a list of approvals
+	 * This function returns a list of all approvals seperated by individual
+	 * and group approvals. If the document id
+	 * is passed, then only this document will be checked for approvals. The
+	 * same is true for the version of a document which limits the list
+	 * further.
+	 *
+	 * The result array has two elements:
+	 * - indstatus: which contains the approvals by individuals (users)
+	 * - grpstatus: which contains the approvals by groups
+	 *
+	 * Each element is itself an array of approvals with the following elements:
+	 * - approveID: unique id of approval
+	 * - documentID: id of document, that needs to be approved
+	 * - version: version of document, that needs to be approved
+	 * - type: 0 for individual approval, 1 for group approval
+	 * - required: id of user who is required to do the approval
+	 * - status: 0 not approved, ....
+	 * - comment: comment given during approval
+	 * - date: date of approval
+	 * - userID: id of user who has done the approval
+	 *
+	 * @param int $documentID optional document id for which to retrieve the
+	 *        approvals
+	 * @param int $version optional version of the document
+	 * @return array list of all approvals
+	 */
 	function getApprovalStatus($documentID=null, $version=null) { /* {{{ */
 		$db = $this->_dms->getDB();
 
+/*
 		if (!$db->createTemporaryTable("ttapproveid")) {
 			return false;
 		}
-
+*/
 		$status = array("indstatus"=>array(), "grpstatus"=>array());
 
 		// See if the user is assigned as an individual approver.
+		/*
 		$queryStr = "SELECT `tblDocumentApprovers`.*, `tblDocumentApproveLog`.`status`, ".
 			"`tblDocumentApproveLog`.`comment`, `tblDocumentApproveLog`.`date`, ".
 			"`tblDocumentApproveLog`.`userID` ".
@@ -589,6 +658,19 @@ class LetoDMS_Core_User {
 			($version==null ? "" : "AND `tblDocumentApprovers`.`version` = '". $version ."' ").
 			"AND `tblDocumentApprovers`.`type`='0' ".
 			"AND `tblDocumentApprovers`.`required`='". $this->_id ."' ";
+*/
+		$queryStr =
+   "SELECT `tblDocumentApprovers`.*, `tblDocumentApproveLog`.`status`, ".
+			"`tblDocumentApproveLog`.`comment`, `tblDocumentApproveLog`.`date`, ".
+			"`tblDocumentApproveLog`.`userID` ".
+			"FROM `tblDocumentApprovers` ".
+			"LEFT JOIN `tblDocumentApproveLog` USING (`approveID`) ".
+			"WHERE `tblDocumentApprovers`.`type`='0' ".
+			($documentID==null ? "" : "AND `tblDocumentApprovers`.`documentID` = '". (int) $documentID ."' ").
+			($version==null ? "" : "AND `tblDocumentApprovers`.`version` = '". (int) $version ."' ").
+			"AND `tblDocumentApprovers`.`required`='". $this->_id ."' ".
+			"ORDER BY `tblDocumentApproveLog`.`approveLogID` DESC LIMIT 1";
+
 		$resArr = $db->getResultArray($queryStr);
 		if (is_bool($resArr) && $resArr == false)
 			return false;
@@ -599,6 +681,7 @@ class LetoDMS_Core_User {
 
 		// See if the user is the member of a group that has been assigned to
 		// approve the document version.
+		/*
 		$queryStr = "SELECT `tblDocumentApprovers`.*, `tblDocumentApproveLog`.`status`, ".
 			"`tblDocumentApproveLog`.`comment`, `tblDocumentApproveLog`.`date`, ".
 			"`tblDocumentApproveLog`.`userID` ".
@@ -611,6 +694,19 @@ class LetoDMS_Core_User {
 			($version==null ? "" : "AND `tblDocumentApprovers`.`version` = '". $version ."' ").
 			"AND `tblDocumentApprovers`.`type`='1' ".
 			"AND `tblGroupMembers`.`userID`='". $this->_id ."'";
+			*/
+		$queryStr =
+			"SELECT `tblDocumentApprovers`.*, `tblDocumentApproveLog`.`status`, ".
+			"`tblDocumentApproveLog`.`comment`, `tblDocumentApproveLog`.`date`, ".
+			"`tblDocumentApproveLog`.`userID` ".
+			"FROM `tblDocumentApprovers` ".
+			"LEFT JOIN `tblDocumentApproveLog` USING (`approveID`) ".
+			"LEFT JOIN `tblGroupMembers` ON `tblGroupMembers`.`groupID` = `tblDocumentApprovers`.`required` ".
+			"WHERE `tblDocumentApprovers`.`type`='1' ".
+			($documentID==null ? "" : "AND `tblDocumentApprovers`.`documentID` = '". (int) $documentID ."' ").
+			($version==null ? "" : "AND `tblDocumentApprovers`.`version` = '". (int) $version ."' ").
+			"AND `tblGroupMembers`.`userID`='". $this->_id ."' ".
+			"ORDER BY `tblDocumentApproveLog`.`approveLogID` DESC LIMIT 1";
 		$resArr = $db->getResultArray($queryStr);
 		if (is_bool($resArr) && $resArr == false)
 			return false;
@@ -621,6 +717,16 @@ class LetoDMS_Core_User {
 		return $status;
 	} /* }}} */
 
+	/**
+	 * Get a list of mandatory reviewers
+	 * A user which isn't trusted completely may have assigned mandatory
+	 * reviewers (both users and groups).
+	 * Whenever the user inserts a new document the mandatory reviewers are
+	 * filled in as reviewers.
+	 *
+	 * @return array list of arrays with two elements containing the user id
+	 *         (reviewerUserID) and group id (reviewerGroupID) of the reviewer.
+	 */
 	function getMandatoryReviewers() { /* {{{ */
 		$db = $this->_dms->getDB();
 
@@ -630,6 +736,13 @@ class LetoDMS_Core_User {
 		return $resArr;
 	} /* }}} */
 
+	/**
+	 * Get a list of mandatory approvers
+	 * See {link LetoDMS_User::getMandatoryReviewers}
+	 *
+	 * @return array list of arrays with two elements containing the user id
+	 *         (approverUserID) and group id (approverGroupID) of the approver.
+	 */
 	function getMandatoryApprovers() { /* {{{ */
 		$db = $this->_dms->getDB();
 
@@ -639,6 +752,14 @@ class LetoDMS_Core_User {
 		return $resArr;
 	} /* }}} */
 
+	/**
+	 * Set a mandatory reviewer
+	 * This function sets a mandatory reviewer if it isn't already set.
+	 *
+	 * @param integer $id id of reviewer
+	 * @param boolean $isgroup true if $id is a group
+	 * @return boolean true on success, otherwise false
+	 */
 	function setMandatoryReviewer($id, $isgroup=false) { /* {{{ */
 		$db = $this->_dms->getDB();
 
@@ -646,7 +767,7 @@ class LetoDMS_Core_User {
 
 			$queryStr = "SELECT * FROM tblMandatoryReviewers WHERE userID = " . $this->_id . " AND reviewerGroupID = " . $id;
 			$resArr = $db->getResultArray($queryStr);
-			if (count($resArr)!=0) return;
+			if (count($resArr)!=0) return true;
 
 			$queryStr = "INSERT INTO tblMandatoryReviewers (userID, reviewerGroupID) VALUES (" . $this->_id . ", " . $id .")";
 			$resArr = $db->getResult($queryStr);
@@ -656,7 +777,7 @@ class LetoDMS_Core_User {
 
 			$queryStr = "SELECT * FROM tblMandatoryReviewers WHERE userID = " . $this->_id . " AND reviewerUserID = " . $id;
 			$resArr = $db->getResultArray($queryStr);
-			if (count($resArr)!=0) return;
+			if (count($resArr)!=0) return true;
 
 			$queryStr = "INSERT INTO tblMandatoryReviewers (userID, reviewerUserID) VALUES (" . $this->_id . ", " . $id .")";
 			$resArr = $db->getResult($queryStr);
@@ -665,12 +786,20 @@ class LetoDMS_Core_User {
 
 	} /* }}} */
 
+	/**
+	 * Set a mandatory approver
+	 * This function sets a mandatory approver if it isn't already set.
+	 *
+	 * @param integer $id id of approver
+	 * @param boolean $isgroup true if $id is a group
+	 * @return boolean true on success, otherwise false
+	 */
 	function setMandatoryApprover($id, $isgroup=false) { /* {{{ */
 		$db = $this->_dms->getDB();
 
 		if ($isgroup){
 
-			$queryStr = "SELECT * FROM tblMandatoryApprovers WHERE userID = " . $this->_id . " AND approverGroupID = " . $id;
+			$queryStr = "SELECT * FROM tblMandatoryApprovers WHERE userID = " . $this->_id . " AND approverGroupID = " . (int) $id;
 			$resArr = $db->getResultArray($queryStr);
 			if (count($resArr)!=0) return;
 
@@ -680,7 +809,7 @@ class LetoDMS_Core_User {
 
 		}else{
 
-			$queryStr = "SELECT * FROM tblMandatoryApprovers WHERE userID = " . $this->_id . " AND approverUserID = " . $id;
+			$queryStr = "SELECT * FROM tblMandatoryApprovers WHERE userID = " . $this->_id . " AND approverUserID = " . (int) $id;
 			$resArr = $db->getResultArray($queryStr);
 			if (count($resArr)!=0) return;
 
@@ -690,17 +819,30 @@ class LetoDMS_Core_User {
 		}
 	} /* }}} */
 
+	/**
+	 * Deletes all mandatory reviewers
+	 *
+	 * @return boolean true on success, otherwise false
+	 */
 	function delMandatoryReviewers() { /* {{{ */
 		$db = $this->_dms->getDB();
 		$queryStr = "DELETE FROM tblMandatoryReviewers WHERE userID = " . $this->_id;
 		if (!$db->getResult($queryStr)) return false;
+		return true;
 	} /* }}} */
 
+	/**
+	 * Deletes all mandatory approvers
+	 *
+	 * @return boolean true on success, otherwise false
+	 */
 	function delMandatoryApprovers() { /* {{{ */
 		$db = $this->_dms->getDB();
 
 		$queryStr = "DELETE FROM tblMandatoryApprovers WHERE userID = " . $this->_id;
 		if (!$db->getResult($queryStr)) return false;
+		return true;
 	} /* }}} */
+
 }
 ?>
